@@ -5,15 +5,12 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
-{-# LANGUAGE GeneralizedNewtypeDeriving, TypeFamilies, MultiParamTypeClasses #-}
+{-# LANGUAGE CPP, GeneralizedNewtypeDeriving, PackageImports #-}
 
 module Annex (
 	Annex,
 	AnnexState(..),
-	FileInfo(..),
-	PreferredContentMap,
 	new,
-	newState,
 	run,
 	eval,
 	getState,
@@ -36,16 +33,19 @@ module Annex (
 
 import "mtl" Control.Monad.Reader
 import "MonadCatchIO-transformers" Control.Monad.CatchIO
-import System.Posix.Types (Fd)
 import Control.Concurrent
 
 import Common
 import qualified Git
 import qualified Git.Config
+import Annex.Direct.Fixup
 import Git.CatFile
 import Git.CheckAttr
+import Git.CheckIgnore
 import Git.SharedRepository
+import qualified Git.Hook
 import qualified Git.Queue
+import Types.Key
 import Types.Backend
 import Types.GitConfig
 import qualified Types.Remote
@@ -55,9 +55,17 @@ import Types.TrustLevel
 import Types.Group
 import Types.Messages
 import Types.UUID
-import qualified Utility.Matcher
+import Types.FileMatcher
+import Types.NumCopies
+import Types.LockPool
+import Types.MetaData
+import Types.DesktopNotify
+import Types.CleanupActions
 import qualified Data.Map as M
 import qualified Data.Set as S
+#ifdef WITH_QUVI
+import Utility.Quvi (QuviVersion)
+#endif
 
 {- git-annex's monad is a ReaderT around an AnnexState stored in a MVar.
  - This allows modifying the state in an exception-safe fashion.
@@ -73,63 +81,75 @@ newtype Annex a = Annex { runAnnex :: ReaderT (MVar AnnexState) IO a }
 		Applicative
 	)
 
-type Matcher a = Either [Utility.Matcher.Token a] (Utility.Matcher.Matcher a)
-
-data FileInfo = FileInfo
-	{ relFile :: FilePath -- may be relative to cwd
-	, matchFile :: FilePath -- filepath to match on; may be relative to top
-	}
-
-type PreferredContentMap = M.Map UUID (Utility.Matcher.Matcher (S.Set UUID -> FileInfo -> Annex Bool))
-
 -- internal state storage
 data AnnexState = AnnexState
 	{ repo :: Git.Repo
 	, gitconfig :: GitConfig
 	, backends :: [BackendA Annex]
 	, remotes :: [Types.Remote.RemoteA Annex]
+	, remoteannexstate :: M.Map UUID AnnexState
 	, output :: MessageState
 	, force :: Bool
 	, fast :: Bool
 	, auto :: Bool
+	, daemon :: Bool
 	, branchstate :: BranchState
 	, repoqueue :: Maybe Git.Queue.Queue
 	, catfilehandles :: M.Map FilePath CatFileHandle
 	, checkattrhandle :: Maybe CheckAttrHandle
+	, checkignorehandle :: Maybe (Maybe CheckIgnoreHandle)
 	, forcebackend :: Maybe String
-	, limit :: Matcher (FileInfo -> Annex Bool)
+	, globalnumcopies :: Maybe NumCopies
+	, forcenumcopies :: Maybe NumCopies
+	, limit :: ExpandableMatcher Annex
 	, uuidmap :: Maybe UUIDMap
-	, preferredcontentmap :: Maybe PreferredContentMap
+	, preferredcontentmap :: Maybe (FileMatcherMap Annex)
+	, requiredcontentmap :: Maybe (FileMatcherMap Annex)
 	, shared :: Maybe SharedRepository
 	, forcetrust :: TrustMap
 	, trustmap :: Maybe TrustMap
 	, groupmap :: Maybe GroupMap
 	, ciphers :: M.Map StorableCipher Cipher
-	, lockpool :: M.Map FilePath Fd
+	, lockpool :: LockPool
 	, flags :: M.Map String Bool
 	, fields :: M.Map String String
-	, cleanup :: M.Map String (Annex ())
+	, modmeta :: [ModMeta]
+	, cleanup :: M.Map CleanupAction (Annex ())
 	, inodeschanged :: Maybe Bool
+	, useragent :: Maybe String
+	, errcounter :: Integer
+	, unusedkeys :: Maybe (S.Set Key)
+#ifdef WITH_QUVI
+	, quviversion :: Maybe QuviVersion
+#endif
+	, existinghooks :: M.Map Git.Hook.Hook Bool
+	, desktopnotify :: DesktopNotify
 	}
 
-newState :: Git.Repo -> AnnexState
-newState gitrepo = AnnexState
-	{ repo = gitrepo
-	, gitconfig = extractGitConfig gitrepo
+newState :: GitConfig -> Git.Repo -> AnnexState
+newState c r = AnnexState
+	{ repo = r
+	, gitconfig = c
 	, backends = []
 	, remotes = []
+	, remoteannexstate = M.empty
 	, output = defaultMessageState
 	, force = False
 	, fast = False
 	, auto = False
+	, daemon = False
 	, branchstate = startBranchState
 	, repoqueue = Nothing
 	, catfilehandles = M.empty
 	, checkattrhandle = Nothing
+	, checkignorehandle = Nothing
 	, forcebackend = Nothing
-	, limit = Left []
+	, globalnumcopies = Nothing
+	, forcenumcopies = Nothing
+	, limit = BuildingMatcher []
 	, uuidmap = Nothing
 	, preferredcontentmap = Nothing
+	, requiredcontentmap = Nothing
 	, shared = Nothing
 	, forcetrust = M.empty
 	, trustmap = Nothing
@@ -138,14 +158,26 @@ newState gitrepo = AnnexState
 	, lockpool = M.empty
 	, flags = M.empty
 	, fields = M.empty
+	, modmeta = []
 	, cleanup = M.empty
 	, inodeschanged = Nothing
+	, useragent = Nothing
+	, errcounter = 0
+	, unusedkeys = Nothing
+#ifdef WITH_QUVI
+	, quviversion = Nothing
+#endif
+	, existinghooks = M.empty
+	, desktopnotify = mempty
 	}
 
 {- Makes an Annex state object for the specified git repo.
  - Ensures the config is read, if it was not already. -}
 new :: Git.Repo -> IO AnnexState
-new = newState <$$> Git.Config.read
+new r = do
+	r' <- Git.Config.read r
+	let c = extractGitConfig r'
+	newState c <$> if annexDirect c then fixupDirect r' else return r'
 
 {- Performs an action in the Annex monad from a starting state,
  - returning a new state. -}
@@ -185,9 +217,9 @@ setField field value = changeState $ \s ->
 	s { fields = M.insertWith' const field value $ fields s }
 
 {- Adds a cleanup action to perform. -}
-addCleanup :: String -> Annex () -> Annex ()
-addCleanup uid a = changeState $ \s ->
-	s { cleanup = M.insertWith' const uid a $ cleanup s }
+addCleanup :: CleanupAction -> Annex () -> Annex ()
+addCleanup k a = changeState $ \s ->
+	s { cleanup = M.insertWith' const k a $ cleanup s }
 
 {- Sets the type of output to emit. -}
 setOutput :: OutputType -> Annex ()
