@@ -10,10 +10,10 @@
 module Assistant.WebApp.DashBoard where
 
 import Assistant.WebApp.Common
-import Assistant.WebApp.Utility
 import Assistant.WebApp.RepoList
 import Assistant.WebApp.Notifications
 import Assistant.TransferQueue
+import Assistant.TransferSlots
 import Assistant.DaemonStatus
 import Utility.NotificationBroadcaster
 import Logs.Transfer
@@ -23,15 +23,14 @@ import Types.Key
 import qualified Remote
 import qualified Git
 
-import Text.Hamlet
+import qualified Text.Hamlet as Hamlet
 import qualified Data.Map as M
 import Control.Concurrent
 
 {- A display of currently running and queued transfers. -}
-transfersDisplay :: Bool -> Widget
-transfersDisplay warnNoScript = do
-	webapp <- lift getYesod
-	current <- lift $ M.toList <$> getCurrentTransfers
+transfersDisplay :: Widget
+transfersDisplay = do
+	current <- liftAssistant $ M.toList <$> getCurrentTransfers
 	queued <- take 10 <$> liftAssistant getTransferQueue
 	autoUpdate ident NotifierTransfersR (10 :: Int) (10 :: Int)
 	let transfers = simplifyTransfers $ current ++ queued
@@ -52,7 +51,7 @@ simplifyTransfers [] = []
 simplifyTransfers (x:[]) = [x]
 simplifyTransfers (v@(t1, _):r@((t2, _):l))
 	| equivilantTransfer t1 t2 = simplifyTransfers (v:l)
-	| otherwise = v : (simplifyTransfers r)
+	| otherwise = v : simplifyTransfers r
 
 {- Called by client to get a display of currently in process transfers.
  -
@@ -62,23 +61,23 @@ simplifyTransfers (v@(t1, _):r@((t2, _):l))
  - body is. To get the widget head content, the widget is also 
  - inserted onto the getDashboardR page.
  -}
-getTransfersR :: NotificationId -> Handler RepHtml
+getTransfersR :: NotificationId -> Handler Html
 getTransfersR nid = do
 	waitNotifier getTransferBroadcaster nid
 
-	p <- widgetToPageContent $ transfersDisplay False
-	hamletToRepHtml $ [hamlet|^{pageBody p}|]
+	p <- widgetToPageContent transfersDisplay
+	giveUrlRenderer $ [hamlet|^{pageBody p}|]
 
 {- The main dashboard. -}
 dashboard :: Bool -> Widget
 dashboard warnNoScript = do
 	let repolist = repoListDisplay $
 		mainRepoSelector { nudgeAddMore = True }
-	let transferlist = transfersDisplay warnNoScript
+	let transferlist = transfersDisplay
 	$(widgetFile "dashboard/main")
 
-getDashboardR :: Handler RepHtml
-getDashboardR = ifM (inFirstRun)
+getDashboardR :: Handler Html
+getDashboardR = ifM inFirstRun
 	( redirect ConfigurationR
 	, page "" (Just DashBoard) $ dashboard True
 	)
@@ -88,16 +87,15 @@ headDashboardR :: Handler ()
 headDashboardR = noop
 
 {- Same as DashboardR, except no autorefresh at all (and no noscript warning). -}
-getNoScriptR :: Handler RepHtml
+getNoScriptR :: Handler Html
 getNoScriptR = page "" (Just DashBoard) $ dashboard False
 
 {- Same as DashboardR, except with autorefreshing via meta refresh. -}
-getNoScriptAutoR :: Handler RepHtml
+getNoScriptAutoR :: Handler Html
 getNoScriptAutoR = page "" (Just DashBoard) $ do
-	let ident = NoScriptR
 	let delayseconds = 3 :: Int
 	let this = NoScriptAutoR
-	toWidgetHead $(hamletFile $ hamletTemplate "dashboard/metarefresh")
+	toWidgetHead $(Hamlet.hamletFile $ hamletTemplate "dashboard/metarefresh")
 	dashboard False
 
 {- The javascript code does a post. -}
@@ -107,7 +105,7 @@ postFileBrowserR = void openFileBrowser
 {- Used by non-javascript browsers, where clicking on the link actually
  - opens this page, so we redirect back to the referrer. -}
 getFileBrowserR :: Handler ()
-getFileBrowserR = whenM openFileBrowser $ redirectBack
+getFileBrowserR = whenM openFileBrowser redirectBack
 
 {- Opens the system file browser on the repo, or, as a fallback,
  - goes to a file:// url. Returns True if it's ok to redirect away
@@ -118,33 +116,57 @@ getFileBrowserR = whenM openFileBrowser $ redirectBack
 openFileBrowser :: Handler Bool
 openFileBrowser = do
 	path <- liftAnnex $ fromRepo Git.repoPath
-	ifM (liftIO $ inPath cmd <&&> inPath cmd)
+#ifdef darwin_HOST_OS
+	let cmd = "open"
+	let params = [Param path]
+#else
+#ifdef mingw32_HOST_OS
+	let cmd = "cmd"
+	let params = [Param $ "/c start " ++ path]
+#else
+	let cmd = "xdg-open"
+	let params = [Param path]
+#endif
+#endif
+	ifM (liftIO $ inPath cmd)
 		( do
-			void $ liftIO $ forkIO $ void $
-				boolSystem cmd [Param path]
+			let run = void $ liftIO $ forkIO $ void $
+				boolSystem cmd params
+			run
+#ifdef mingw32_HOST_OS
+			{- On windows, if the file browser is not
+			 - already open, it comes up below the
+			 - web browser when started. 
+			 -
+			 - Running it a second time brings it
+			 - to the foreground.
+			 -
+			 - Seems to need a delay long enough for the file
+			 - browser to be open in order to work. Here 1
+			 - second. -}
+			liftIO $ threadDelay 1000000
+			run
+#endif
 			return True
 		, do
 			void $ redirect $ "file://" ++ path
 			return False
 		)
-  where
-#ifdef darwin_HOST_OS
-	cmd = "open"
-#else
-	cmd = "xdg-open"
-#endif
 
 {- Transfer controls. The GET is done in noscript mode and redirects back
  - to the referring page. The POST is called by javascript. -}
 getPauseTransferR :: Transfer -> Handler ()
-getPauseTransferR t = pauseTransfer t >> redirectBack
+getPauseTransferR = noscript postPauseTransferR
 postPauseTransferR :: Transfer -> Handler ()
-postPauseTransferR t = pauseTransfer t
+postPauseTransferR = liftAssistant . pauseTransfer
 getStartTransferR :: Transfer -> Handler ()
-getStartTransferR t = startTransfer t >> redirectBack
+getStartTransferR = noscript postStartTransferR
 postStartTransferR :: Transfer -> Handler ()
-postStartTransferR t = startTransfer t
+postStartTransferR = liftAssistant . startTransfer
 getCancelTransferR :: Transfer -> Handler ()
-getCancelTransferR t = cancelTransfer False t >> redirectBack
+getCancelTransferR = noscript postCancelTransferR
 postCancelTransferR :: Transfer -> Handler ()
-postCancelTransferR t = cancelTransfer False t
+postCancelTransferR = liftAssistant . cancelTransfer False
+
+noscript :: (Transfer -> Handler ()) -> Transfer -> Handler ()
+noscript a t = a t >> redirectBack
