@@ -5,7 +5,7 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
-{-# LANGUAGE TypeFamilies, QuasiQuotes, MultiParamTypeClasses, TemplateHaskell, OverloadedStrings, RankNTypes #-}
+{-# LANGUAGE QuasiQuotes, TemplateHaskell, OverloadedStrings #-}
 
 module Assistant.WebApp.Documentation where
 
@@ -21,12 +21,12 @@ licenseFile = do
 	base <- standaloneAppBase
 	return $ (</> "LICENSE") <$> base
 
-getAboutR :: Handler RepHtml
+getAboutR :: Handler Html
 getAboutR = page "About git-annex" (Just About) $ do
 	builtinlicense <- isJust <$> liftIO licenseFile
 	$(widgetFile "documentation/about")
 
-getLicenseR :: Handler RepHtml
+getLicenseR :: Handler Html
 getLicenseR = do
 	v <- liftIO licenseFile
 	case v of
@@ -37,6 +37,6 @@ getLicenseR = do
 			license <- liftIO $ readFile f
 			$(widgetFile "documentation/license")
 
-getRepoGroupR :: Handler RepHtml
-getRepoGroupR = page "About repository groups" (Just About) $ do
+getRepoGroupR :: Handler Html
+getRepoGroupR = page "About repository groups" (Just About) $
 	$(widgetFile "documentation/repogroup")
