@@ -5,7 +5,7 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
-{-# LANGUAGE TypeFamilies, QuasiQuotes, MultiParamTypeClasses, TemplateHaskell, OverloadedStrings, RankNTypes #-}
+{-# LANGUAGE QuasiQuotes, TemplateHaskell, OverloadedStrings #-}
 
 module Assistant.WebApp.SideBar where
 
@@ -18,7 +18,6 @@ import Assistant.DaemonStatus
 import Utility.NotificationBroadcaster
 import Utility.Yesod
 
-import Yesod
 import Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Map as M
@@ -28,7 +27,7 @@ sideBarDisplay :: Widget
 sideBarDisplay = do
 	let content = do
 		{- Add newest alerts to the sidebar. -}
-		alertpairs <- lift $ M.toList . alertMap
+		alertpairs <- liftH $ M.toList . alertMap
 			<$> liftAssistant getDaemonStatus
 		mapM_ renderalert $
 			take displayAlerts $ reverse $ sortAlertPairs alertpairs
@@ -51,6 +50,7 @@ sideBarDisplay = do
 		let message = renderAlertMessage alert
 		let messagelines = T.lines message
 		let multiline = length messagelines > 1
+		let buttons = zip (alertButtons alert) [1..]
 		$(widgetFile "sidebar/alert")
 
 {- Called by client to get a sidebar display.
@@ -61,7 +61,7 @@ sideBarDisplay = do
  - body is. To get the widget head content, the widget is also 
  - inserted onto all pages.
  -}
-getSideBarR :: NotificationId -> Handler RepHtml
+getSideBarR :: NotificationId -> Handler Html
 getSideBarR nid = do
 	waitNotifier getAlertBroadcaster nid
 
@@ -73,33 +73,38 @@ getSideBarR nid = do
 	liftIO $ threadDelay 100000
 
 	page <- widgetToPageContent sideBarDisplay
-	hamletToRepHtml $ [hamlet|^{pageBody page}|]
+	giveUrlRenderer $ [hamlet|^{pageBody page}|]
 
 {- Called by the client to close an alert. -}
 getCloseAlert :: AlertId -> Handler ()
 getCloseAlert = liftAssistant . removeAlert
 
 {- When an alert with a button is clicked on, the button takes us here. -}
-getClickAlert :: AlertId -> Handler ()
-getClickAlert i = do
+getClickAlert :: AlertId -> Int -> Handler ()
+getClickAlert i bnum = do
 	m <- alertMap <$> liftAssistant getDaemonStatus
 	case M.lookup i m of
-		Just (Alert { alertButton = Just b }) -> do
-			{- Spawn a thread to run the action while redirecting. -}
-			case buttonAction b of
-				Nothing -> noop
-				Just a -> liftIO $ void $ forkIO $ a i
-			redirect $ buttonUrl b
+		Just (Alert { alertButtons = bs })
+			| length bs >= bnum -> do
+				let b = bs !! (bnum - 1)
+				{- Spawn a thread to run the action
+				 - while redirecting. -}
+				case buttonAction b of
+					Nothing -> noop
+					Just a -> liftIO $ void $ forkIO $ a i
+				redirect $ buttonUrl b
+			| otherwise -> redirectBack
 		_ -> redirectBack
 
-htmlIcon :: AlertIcon -> GWidget WebApp WebApp ()
+htmlIcon :: AlertIcon -> Widget
 htmlIcon ActivityIcon = [whamlet|<img src="@{StaticR activityicon_gif}" alt="">|]
 htmlIcon SyncIcon = [whamlet|<img src="@{StaticR syncicon_gif}" alt="">|]
 htmlIcon InfoIcon = bootstrapIcon "info-sign"
 htmlIcon SuccessIcon = bootstrapIcon "ok"
 htmlIcon ErrorIcon = bootstrapIcon "exclamation-sign"
+htmlIcon UpgradeIcon = bootstrapIcon "arrow-up"
 -- utf-8 umbrella (utf-8 cloud looks too stormy)
 htmlIcon TheCloud = [whamlet|&#9730;|]
 
-bootstrapIcon :: Text -> GWidget sub master ()
+bootstrapIcon :: Text -> Widget
 bootstrapIcon name = [whamlet|<i .icon-#{name}></i>|]
