@@ -7,7 +7,8 @@
 
 {-# LANGUAGE TypeFamilies, QuasiQuotes, MultiParamTypeClasses #-}
 {-# LANGUAGE TemplateHaskell, OverloadedStrings, RankNTypes #-}
-{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE FlexibleInstances, FlexibleContexts #-}
+{-# LANGUAGE CPP #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Assistant.WebApp.Types where
@@ -20,9 +21,12 @@ import Utility.NotificationBroadcaster
 import Utility.WebApp
 import Utility.Yesod
 import Logs.Transfer
+import Utility.Gpg (KeyId)
 import Build.SysConfig (packageversion)
+import Types.ScheduledActivity
+import Assistant.WebApp.RepoId
+import Types.Distribution
 
-import Yesod
 import Yesod.Static
 import Text.Hamlet
 import Data.Text (Text, pack, unpack)
@@ -37,21 +41,22 @@ mkYesodData "WebApp" $(parseRoutesFile "Assistant/WebApp/routes")
 
 data WebApp = WebApp
 	{ assistantData :: AssistantData
-	, secretToken :: Text
+	, authToken :: AuthToken
 	, relDir :: Maybe FilePath
 	, getStatic :: Static
 	, postFirstRun :: Maybe (IO String)
+	, cannotRun :: Maybe String
 	, noAnnex :: Bool
 	, listenHost ::Maybe HostName
 	}
 
 instance Yesod WebApp where
 	{- Require an auth token be set when accessing any (non-static) route -}
-	isAuthorized _ _ = checkAuthToken secretToken
+	isAuthorized _ _ = checkAuthToken authToken
 
 	{- Add the auth token to every url generated, except static subsite
 	 - urls (which can show up in Permission Denied pages). -}
-	joinPath = insertAuthToken secretToken excludeStatic
+	joinPath = insertAuthToken authToken excludeStatic
 	  where
 		excludeStatic [] = True
 		excludeStatic (p:_) = p /= "static"
@@ -68,10 +73,10 @@ instance Yesod WebApp where
 	defaultLayout content = do
 		webapp <- getYesod
 		pageinfo <- widgetToPageContent $ do
-			addStylesheet $ StaticR css_bootstrap_css
-			addStylesheet $ StaticR css_bootstrap_responsive_css
+			addStylesheet $ StaticR bootstrap_css
+			addStylesheet $ StaticR bootstrap_responsive_css
 			$(widgetFile "error")
-		hamletToRepHtml $(hamletFile $ hamletTemplate "bootstrap")
+		giveUrlRenderer $(hamletFile $ hamletTemplate "bootstrap")
 
 instance RenderMessage WebApp FormMessage where
 	renderMessage _ _ = defaultFormMessage
@@ -81,29 +86,65 @@ instance RenderMessage WebApp FormMessage where
  - When the webapp is run outside a git-annex repository, the fallback
  - value is returned.
  -}
+#if MIN_VERSION_yesod(1,2,0)
+liftAnnexOr :: forall a. a -> Annex a -> Handler a
+#else
 liftAnnexOr :: forall sub a. a -> Annex a -> GHandler sub WebApp a
+#endif
 liftAnnexOr fallback a = ifM (noAnnex <$> getYesod)
 	( return fallback
 	, liftAssistant $ liftAnnex a
 	)
 
+#if MIN_VERSION_yesod(1,2,0)
+instance LiftAnnex Handler where
+#else
 instance LiftAnnex (GHandler sub WebApp) where
-	liftAnnex = liftAnnexOr $ error "internal runAnnex"
+#endif
+	liftAnnex = liftAnnexOr $ error "internal liftAnnex"
 
+#if MIN_VERSION_yesod(1,2,0)
+instance LiftAnnex (WidgetT WebApp IO) where
+#else
 instance LiftAnnex (GWidget WebApp WebApp) where
-	liftAnnex = lift . liftAnnex
+#endif
+	liftAnnex = liftH . liftAnnex
 
 class LiftAssistant m where
 	liftAssistant :: Assistant a -> m a
 
+#if MIN_VERSION_yesod(1,2,0)
+instance LiftAssistant Handler where
+#else
 instance LiftAssistant (GHandler sub WebApp) where
+#endif
 	liftAssistant a = liftIO . flip runAssistant a
 		=<< assistantData <$> getYesod
 
+#if MIN_VERSION_yesod(1,2,0)
+instance LiftAssistant (WidgetT WebApp IO) where
+#else
 instance LiftAssistant (GWidget WebApp WebApp) where
-	liftAssistant = lift . liftAssistant
+#endif
+	liftAssistant = liftH . liftAssistant
 
-type Form x = Html -> MForm WebApp WebApp (FormResult x, Widget)
+#if MIN_VERSION_yesod(1,2,0)
+type MkMForm x = MForm Handler (FormResult x, Widget)
+#else
+type MkMForm x = MForm WebApp WebApp (FormResult x, Widget)
+#endif
+
+#if MIN_VERSION_yesod(1,2,0)
+type MkAForm x = AForm Handler x
+#else
+type MkAForm x = AForm WebApp WebApp x
+#endif
+
+#if MIN_VERSION_yesod(1,2,0)
+type MkField x = Monad m => RenderMessage (HandlerSite m) FormMessage => Field m x
+#else
+type MkField x = RenderMessage master FormMessage => Field sub master x
+#endif
 
 data RepoSelector = RepoSelector
 	{ onlyCloud :: Bool
@@ -113,9 +154,6 @@ data RepoSelector = RepoSelector
 	}
 	deriving (Read, Show, Eq)
 
-data RepoListNotificationId = RepoListNotificationId NotificationId RepoSelector
-	deriving (Read, Show, Eq)
-
 data RemovableDrive = RemovableDrive 
 	{ diskFree :: Maybe Integer
 	, mountPoint :: Text
@@ -123,12 +161,10 @@ data RemovableDrive = RemovableDrive
 	}
 	deriving (Read, Show, Eq, Ord)
 
-{- Only needed to work around old-yesod bug that emits a warning message
- - when a route has two parameters. -}
-data FilePathAndUUID = FilePathAndUUID FilePath UUID
-	deriving (Read, Show, Eq)
+data RepoKey = RepoKey KeyId | NoRepoKey
+	deriving (Read, Show, Eq, Ord)
 
-instance PathPiece FilePathAndUUID where
+instance PathPiece Bool where
 	toPathPiece = pack . show
 	fromPathPiece = readish . unpack
 
@@ -136,6 +172,10 @@ instance PathPiece RemovableDrive where
 	toPathPiece = pack . show
 	fromPathPiece = readish . unpack
 
+instance PathPiece RepoKey where
+	toPathPiece = pack . show
+	fromPathPiece = readish . unpack
+
 instance PathPiece SshData where
 	toPathPiece = pack . show
 	fromPathPiece = readish . unpack
@@ -172,10 +212,6 @@ instance PathPiece PairKey where
 	toPathPiece = pack . show
 	fromPathPiece = readish . unpack
 
-instance PathPiece RepoListNotificationId where
-	toPathPiece = pack . show
-	fromPathPiece = readish . unpack
-
 instance PathPiece RepoSelector where
 	toPathPiece = pack . show
 	fromPathPiece = readish . unpack
@@ -183,3 +219,15 @@ instance PathPiece RepoSelector where
 instance PathPiece ThreadName where
 	toPathPiece = pack . show
 	fromPathPiece = readish . unpack
+
+instance PathPiece ScheduledActivity where
+	toPathPiece = pack . show
+	fromPathPiece = readish . unpack
+
+instance PathPiece RepoId where
+	toPathPiece = pack . show
+	fromPathPiece = readish . unpack
+
+instance PathPiece GitAnnexDistribution where
+	toPathPiece = pack . show
+	fromPathPiece = readish . unpack
