@@ -10,11 +10,10 @@ module Backend.URL (
 	fromUrl
 ) where
 
-import Data.Hash.MD5
-
 import Common.Annex
 import Types.Backend
 import Types.Key
+import Backend.Utilities
 
 backends :: [Backend]
 backends = [backend]
@@ -27,16 +26,12 @@ backend = Backend
 	, canUpgradeKey = Nothing
 	}
 
-fromUrl :: String -> Maybe Integer -> Key
-fromUrl url size = stubKey
-	{ keyName = key
-	, keyBackendName = "URL"
-	, keySize = size
-	}
-  where
-	{- when it's not too long, use the url as the key name
-	 - 256 is the absolute filename max, but use a shorter
-	 - length because this is not the entire key filename. -}
-	key
-		| length url < 128 = url
-		| otherwise = take 128 url ++ "-" ++ md5s (Str url)
+{- Every unique url has a corresponding key. -}
+fromUrl :: String -> Maybe Integer -> Annex Key
+fromUrl url size = do
+	n <- genKeyName url
+	return $ stubKey
+		{ keyName = n
+		, keyBackendName = "URL"
+		, keySize = size
+		}
