@@ -3,23 +3,22 @@
 module Build.Configure where
 
 import System.Directory
-import Data.List
-import System.Process
 import Control.Applicative
-import System.FilePath
-import System.Environment
-import Data.Maybe
+import System.Environment (getArgs)
 import Control.Monad.IfElse
+import Control.Monad
 
 import Build.TestConfig
+import Build.Version
 import Utility.SafeCommand
-import Utility.Monad
-import Utility.Exception
 import Utility.ExternalSHA
+import Utility.Env
+import qualified Git.Version
 
 tests :: [TestCase]
 tests =
 	[ TestCase "version" getVersion
+	, TestCase "UPGRADE_LOCATION" getUpgradeLocation
 	, TestCase "git" $ requireCmd "git" "git --version >/dev/null"
 	, TestCase "git version" getGitVersion
 	, testCp "cp_a" "-a"
@@ -30,8 +29,14 @@ tests =
 	, TestCase "curl" $ testCmd "curl" "curl --version >/dev/null"
 	, TestCase "wget" $ testCmd "wget" "wget --version >/dev/null"
 	, TestCase "bup" $ testCmd "bup" "bup --version >/dev/null"
-	, TestCase "gpg" $ testCmd "gpg" "gpg --version >/dev/null"
+	, TestCase "nice" $ testCmd "nice" "nice true >/dev/null"
+	, TestCase "ionice" $ testCmd "ionice" "ionice -c3 true >/dev/null"
+	, TestCase "nocache" $ testCmd "nocache" "nocache true >/dev/null"
+	, TestCase "gpg" $ maybeSelectCmd "gpg"
+		[ ("gpg", "--version >/dev/null")
+		, ("gpg2", "--version >/dev/null") ]
 	, TestCase "lsof" $ findCmdPath "lsof" "lsof"
+	, TestCase "git-remote-gcrypt" $ findCmdPath "gcrypt" "git-remote-gcrypt"
 	, TestCase "ssh connection caching" getSshConnectionCaching
 	] ++ shaTestCases
 	[ (1, "da39a3ee5e6b4b0d3255bfef95601890afd80709")
@@ -41,8 +46,8 @@ tests =
 	, (384, "38b060a751ac96384cd9327eb1b1e36a21fdb71114be07434c0cc7bf63f6e1da274edebfe76f65fbd51ad2f14898b95b")
 	]
 
-{- shaNsum are the program names used by coreutils. Some systems like OSX
- - sometimes install these with 'g' prefixes.
+{- shaNsum are the program names used by coreutils. Some systems
+ - install these with 'g' prefixes.
  -
  - On some systems, shaN is used instead, but on other
  - systems, it might be "hashalot", which does not produce
@@ -62,14 +67,9 @@ shaTestCases l = map make l
 				then return $ Just c
 				else search cmds
 	
-	shacmds n = concatMap (\x -> [x, 'g':x, osxpath </> x]) $
+	shacmds n = concatMap (\x -> [x, 'g':x]) $
 		map (\x -> "sha" ++ show n ++ x) ["sum", ""]
 
-	{- Max OSX sometimes puts GNU tools outside PATH, so look in
-	 - the location it uses, and remember where to run them
-	 - from. -}
-	osxpath = "/opt/local/libexec/gnubin"
-
 tmpDir :: String
 tmpDir = "tmp"
 
@@ -82,68 +82,23 @@ testCp k option = TestCase cmd $ testCmd k cmdline
 	cmd = "cp " ++ option
 	cmdline = cmd ++ " " ++ testFile ++ " " ++ testFile ++ ".new"
 
-isReleaseBuild :: IO Bool
-isReleaseBuild = isJust <$> catchMaybeIO (getEnv "RELEASE_BUILD")
-
-{- Version is usually based on the major version from the changelog, 
- - plus the date of the last commit, plus the git rev of that commit.
- - This works for autobuilds, ad-hoc builds, etc.
- -
- - If git or a git repo is not available, or something goes wrong,
- - or this is a release build, just use the version from the changelog. -}
-getVersion :: Test
-getVersion = do
-	changelogversion <- getChangelogVersion
-	version <- ifM (isReleaseBuild)
-		( return changelogversion
-		, catchDefaultIO changelogversion $ do
-			let major = takeWhile (/= '.') changelogversion
-			autoversion <- readProcess "sh"
-				[ "-c"
-				, "git log -n 1 --format=format:'%ci %h'| sed -e 's/-//g' -e 's/ .* /-g/'"
-				] ""
-			if null autoversion
-				then return changelogversion
-				else return $ concat [ major, ".", autoversion ]
-		)
-	return $ Config "packageversion" (StringConfig version)
-	
-getChangelogVersion :: IO String
-getChangelogVersion = do
-	changelog <- readFile "debian/changelog"
-	let verline = takeWhile (/= '\n') changelog
-	return $ middle (words verline !! 1)
-  where
-	middle = drop 1 . init
+getUpgradeLocation :: Test
+getUpgradeLocation = do
+	e <- getEnv "UPGRADE_LOCATION"
+	return $ Config "upgradelocation" $ MaybeStringConfig e
 
 getGitVersion :: Test
 getGitVersion = do
-	s <- readProcess "git" ["--version"] ""
-	let version = unwords $ drop 2 $ words $ head $ lines s
-	return $ Config "gitversion" (StringConfig version)
+	v <- Git.Version.installed
+	let oldestallowed = Git.Version.normalize "1.7.1.0"
+	when (v < oldestallowed) $
+		error $ "installed git version " ++ show v ++ " is too old! (Need " ++ show oldestallowed ++ " or newer)"
+	return $ Config "gitversion" $ StringConfig $ show v
 
 getSshConnectionCaching :: Test
 getSshConnectionCaching = Config "sshconnectioncaching" . BoolConfig <$>
 	boolSystem "sh" [Param "-c", Param "ssh -o ControlPersist=yes -V >/dev/null 2>/dev/null"]
 
-{- Set up cabal file with version. -}
-cabalSetup :: IO ()
-cabalSetup = do
-	version <- takeWhile (/= '~') <$> getChangelogVersion
-	cabal <- readFile cabalfile
-	writeFile tmpcabalfile $ unlines $ 
-		map (setfield "Version" version) $
-		lines cabal
-	renameFile tmpcabalfile cabalfile
-  where
-	cabalfile = "git-annex.cabal"
-	tmpcabalfile = cabalfile++".tmp"
-	setfield field value s
-		| fullfield `isPrefixOf` s = fullfield ++ value
-		| otherwise = s
-	  where
-		fullfield = field ++ ": "
-
 setup :: IO ()
 setup = do
 	createDirectoryIfMissing True tmpDir
@@ -161,8 +116,8 @@ run ts = do
 		then writeSysConfig $ androidConfig config
 		else writeSysConfig config
 	cleanup
-	whenM (isReleaseBuild) $
-		cabalSetup
+	whenM isReleaseBuild $
+		cabalSetup "git-annex.cabal"
 
 {- Hard codes some settings to cross-compile for Android. -}
 androidConfig :: [Config] -> [Config]
@@ -176,4 +131,3 @@ androidConfig c = overrides ++ filter (not . overridden) c
 		]
 	overridden (Config k _) = k `elem` overridekeys
 	overridekeys = map (\(Config k _) -> k) overrides
-
