@@ -34,18 +34,21 @@ import Text.Parsec
 import Text.Parsec.String
 import Control.Applicative ((<$>))
 import Data.Either
-import Data.List
+import Data.List hiding (find)
 import Data.String.Utils
 import Data.Char
 import System.Environment
 import System.FilePath
 import System.Directory
+import System.IO
 import Control.Monad
+import Prelude hiding (log)
 
 import Utility.Monad
 import Utility.Misc
 import Utility.Exception
 import Utility.Path
+import Utility.FileSystemEncoding
 
 data Coord = Coord
 	{ coordLine :: Int
@@ -85,27 +88,27 @@ coordsParser = (try singleline <|> try weird <|> multiline) <?> "Coords"
   where
   	singleline = do
 		line <- number
-		char ':'
+		void $ char ':'
 		startcol <- number
-		char '-'
+		void $ char '-'
 		endcol <- number
 		return $ (Coord line startcol, Coord line endcol)
 
 	weird = do
 		line <- number
-		char ':'
+		void $ char ':'
 		col <- number
 		return $ (Coord line col, Coord line col)
 
 	multiline = do
 		start <- fromparens
-		char '-'
+		void $ char '-'
 		end <- fromparens
 		return $ (start, end)
 
 	fromparens = between (char '(') (char ')') $ do
 		line <- number
-		char ','
+		void $ char ','
 		col <- number
 		return $ Coord line col
 
@@ -121,19 +124,19 @@ indentedLine = indent >> restOfLine
 spliceParser :: Parser Splice
 spliceParser = do
 	file <- many1 (noneOf ":\n")
-	char ':'
+	void $ char ':'
 	(start, end) <- coordsParser
-	string ": Splicing "
+	void $ string ": Splicing "
 	splicetype <- tosplicetype
 		<$> (string "expression" <|> string "declarations")
-	newline
+	void newline
 
 	getthline <- expressionextractor
 	expression <- unlines <$> many1 getthline
 
-	indent
-	string "======>"	
-	newline
+	void indent
+	void $ string "======>"	
+	void newline
 
 	getcodeline <- expressionextractor
 	realcoords <- try (Right <$> getrealcoords file) <|> (Left <$> getcodeline)
@@ -157,7 +160,7 @@ spliceParser = do
 	expressionextractor = do
 		i <- lookAhead indent
 		return $ try $ do
-			string i
+			void $ string i
 			restOfLine
 	
 	{- When splicing declarations, GHC will output a splice
@@ -165,9 +168,9 @@ spliceParser = do
 	 - the first line will give the actual coordinates of the
 	 - line that was spliced. -}
 	getrealcoords file = do
-		indent
-		string file
-		char ':'
+		void indent
+		void $ string file
+		void $ char ':'
 		char '\n' `after` coordsParser
 
 {- Extracts the splices, ignoring the rest of the compiler output. -}
@@ -196,16 +199,20 @@ splicesExtractor = rights <$> many extract
  - starts with "import "
  -}
 applySplices :: FilePath -> Maybe String -> [Splice] -> IO ()
+applySplices _ _ [] = noop
 applySplices destdir imports splices@(first:_) = do
 	let f = splicedFile first
 	let dest = (destdir </> f)
-	lls <- map (++ "\n") . lines <$> readFileStrict f
+	lls <- map (++ "\n") . lines <$> readFileStrictAnyEncoding f
 	createDirectoryIfMissing True (parentDir dest)
 	let newcontent = concat $ addimports $ expand lls splices
-	oldcontent <- catchMaybeIO $ readFileStrict dest
+	oldcontent <- catchMaybeIO $ readFileStrictAnyEncoding dest
 	when (oldcontent /= Just newcontent) $ do
 		putStrLn $ "splicing " ++ f
-		writeFile dest newcontent
+		withFile dest WriteMode $ \h -> do
+		        fileEncoding h
+			hPutStr h newcontent
+		        hClose h
   where
   	expand lls [] = lls
   	expand lls (s:rest)
@@ -237,10 +244,10 @@ expandDeclarationSplice s lls = concat [before, [splice], end]
 
 {- Expression splices are expanded within their line. -}
 expandExpressionSplice :: Splice -> [String] -> [String]
-expandExpressionSplice s lls = concat [before, spliced:padding, end]
+expandExpressionSplice sp lls = concat [before, spliced:padding, end]
   where
-	cs = spliceStart s
-	ce = spliceEnd s
+	cs = spliceStart sp
+	ce = spliceEnd sp
 
 	(before, rest) = splitAt (coordLine cs - 1) lls
 	(oldlines, end) = splitAt (1 + coordLine (offsetCoord ce cs)) rest
@@ -251,7 +258,7 @@ expandExpressionSplice s lls = concat [before, spliced:padding, end]
 		_ -> ([], [], [])
 	spliced = concat
 		[ joinsplice $ deqqstart $ take (coordColumn cs - 1) splicestart
-		, addindent (findindent splicestart) (mangleCode $ splicedCode s)
+		, addindent (findindent splicestart) (mangleCode $ splicedCode sp)
 		, deqqend $ drop (coordColumn ce) spliceend
 		]
 
@@ -260,7 +267,7 @@ expandExpressionSplice s lls = concat [before, spliced:padding, end]
 
 	{- splicing leaves $() quasiquote behind; remove it -}
 	deqqstart s = case reverse s of
-		('(':'$':rest) -> reverse rest
+		('(':'$':restq) -> reverse restq
 		_ -> s
 	deqqend (')':s) = s
 	deqqend s = s
@@ -294,11 +301,14 @@ expandExpressionSplice s lls = concat [before, spliced:padding, end]
 {- Tweaks code output by GHC in splices to actually build. Yipes. -}
 mangleCode :: String -> String
 mangleCode = flip_colon
+	. remove_unnecessary_type_signatures
+	. lambdaparenhack
 	. lambdaparens
 	. declaration_parens
 	. case_layout
 	. case_layout_multiline
 	. yesod_url_render_hack
+	. text_builder_hack
 	. nested_instances 
 	. collapse_multiline_strings
 	. remove_package_version
@@ -328,23 +338,73 @@ mangleCode = flip_colon
 		-- skip lambdas inside tuples or parens
 		prefix <- noneOf "(, \n"
 		preindent <- many1 $ oneOf " \n"
-		string "\\ "
+		void $ string "\\ "
 		lambdaparams <- restofline
-		indent <- many1 $ char ' '
-		string "-> "
+		continuedlambdaparams <- many $ try $ do
+			indent1 <- many1 $ char ' '
+			p <- satisfy isLetter
+			aram <- many $ satisfy isAlphaNum <|> oneOf "_"
+			void newline
+			return $ indent1 ++ p:aram ++ "\n"
+		indent1 <- many1 $ char ' '
+		void $ string "-> "
 		firstline <- restofline
 		lambdalines <- many $ try $ do
-			string indent
-			char ' '
+			void $ string indent1
+			void $ char ' '
 			l <- restofline
-			return $ indent ++ " " ++ l
+			return $ indent1 ++ " " ++ l
 		return $ concat 
 			[ prefix:preindent
 			, "(\\ " ++ lambdaparams ++ "\n"
-			, indent ++ "-> "
+			, concat continuedlambdaparams
+			, indent1 ++ "-> "
 			, lambdaparens $ intercalate "\n" (firstline:lambdalines)
 			, ")\n"
 			]
+	
+	{- Hack to add missing parens in a specific case in yesod
+	 - static route code.
+	 -
+	 -     StaticR
+	 -     yesod_dispatch_env_a4iDV
+	 -     (\ p_a4iE2 r_a4iE3
+	 -        -> r_a4iE3
+	 -          {Network.Wai.pathInfo = p_a4iE2}
+	 -        xrest_a4iDT req_a4iDW)) }
+	 -
+	 - Need to add another paren around the lambda, and close it
+	 - before its parameters. lambdaparens misses this one because
+	 - there is already one paren present.
+	 -
+	 - Note that the { } may be on the same line, or wrapped to next.
+	 -
+	 - FIXME: This is a hack. lambdaparens could just always add a
+	 - layer of parens even when a lambda seems to be in parent.
+	 -}
+	lambdaparenhack = parsecAndReplace $ do
+		indent1 <- many1 $ char ' '
+		staticr <- string "StaticR"
+		void newline
+		void $ string indent1
+		yesod_dispatch_env <- restofline
+		void $ string indent1
+		lambdaprefix <- string "(\\ "
+		l1 <- restofline
+		void $ string indent1
+		lambdaarrow <- string "   ->"
+		l2 <- restofline
+		l3 <- if '{' `elem` l2 && '}' `elem` l2
+			then return ""
+			else do
+				void $ string indent1
+				restofline
+		return $ unlines
+			[ indent1 ++ staticr
+			, indent1 ++ yesod_dispatch_env
+			, indent1 ++ "(" ++ lambdaprefix ++ l1
+			, indent1 ++ lambdaarrow ++ l2 ++ l3 ++ ")"
+			]
 
 	restofline = manyTill (noneOf "\n") newline
 
@@ -355,10 +415,10 @@ mangleCode = flip_colon
 	 - To work around what is likely a GHC bug, just collapse
 	 - multiline strings. -}
 	collapse_multiline_strings = parsecAndReplace $ do
-		string "\\\n"
-		many1 $ oneOf " \t"
-		string "\\"
-		return ""
+		void $ string "\\\n"
+		void $ many1 $ oneOf " \t"
+		void $ string "\\"
+		return "\\n"
 
 	{- GHC outputs splices using explicit braces rather than layout.
 	 - For a case expression, it does something weird:
@@ -386,8 +446,8 @@ mangleCode = flip_colon
 	 - all whitespace up until it.
 	 -}
 	case_layout = parsecAndReplace $ do
-		newline
-		indent <- many1 $ char ' '
+		void newline
+		indent1 <- many1 $ char ' '
 		prefix <- manyTill (noneOf "\n") (try (string "-> "))
 		if length prefix > 10
 			then unexpected "too long a prefix"
@@ -395,24 +455,24 @@ mangleCode = flip_colon
 				then unexpected "lambda expression"
 				else if null prefix
 					then unexpected "second line of lambda"
-					else return $ "\n" ++ indent ++ "; " ++ prefix ++ " -> "
+					else return $ "\n" ++ indent1 ++ "; " ++ prefix ++ " -> "
 	{- Sometimes cases themselves span multiple lines:
 	 -
 	 - Nothing
 	 -   -> foo
 	 -}
 	case_layout_multiline = parsecAndReplace $ do
-		newline
-		indent <- many1 $ char ' '
+		void newline
+		indent1 <- many1 $ char ' '
 		firstline <- restofline
 
-		string indent
+		void $ string indent1
 		indent2 <- many1 $ char ' '
-		string "-> "
+		void $ string "-> "
 		if "\\ " `isInfixOf` firstline
 			then unexpected "lambda expression"
-			else return $ "\n" ++ indent ++ "; " ++ firstline ++ "\n"
-				++ indent ++ indent2 ++ "-> "
+			else return $ "\n" ++ indent1 ++ "; " ++ firstline ++ "\n"
+				++ indent1 ++ indent2 ++ "-> "
 
 	{- (foo, \ -> bar) is not valid haskell, GHC.
 	 - Change to (foo, bar)
@@ -438,6 +498,19 @@ mangleCode = flip_colon
 	 - declarations. -}
 	declaration_parens = replace "StaticR Route Static" "StaticR (Route Static)"
 
+	{- A type signature is sometimes given for an entire lambda,
+	 - which is not properly parenthesized or laid out. This is a
+	 - hack to remove one specific case where this happens and the
+	 - signature is easily inferred, so is just removed.
+	 -}
+	remove_unnecessary_type_signatures = parsecAndReplace $ do
+		void $ string " ::"
+		void newline
+		void $ many1 $ char ' '
+		void $ string "Text.Css.Block Text.Css.Resolved"
+		void newline
+		return ""
+
 	{- GHC may add full package and version qualifications for
 	 - symbols from unimported modules. We don't want these.
 	 -
@@ -454,15 +527,14 @@ mangleCode = flip_colon
 
 	qualifiedSymbol :: Parser String
 	qualifiedSymbol = do
-		s <- token
-		char ':'
+		s <- hstoken
+		void $ char ':'
 		if length s < 5
 			then unexpected "too short to be a namespace"
-			else do
-				token
+			else hstoken
 
-	token :: Parser String
-	token = do
+	hstoken :: Parser String
+	hstoken = do
 		t <- satisfy isLetter
 		oken <- many $ satisfy isAlphaNum <|> oneOf "-.'"
 		return $ t:oken
@@ -495,25 +567,29 @@ mangleCode = flip_colon
  -}
 yesod_url_render_hack :: String -> String
 yesod_url_render_hack = parsecAndReplace $ do
-	string "(toHtml"
-	whitespace
-	string "(\\"
-	whitespace
-	wtf <- token
-	whitespace
-	string "->"
-	whitespace
-	renderer <- token
-	whitespace
-	string wtf
-	whitespace
+	void $ string "(toHtml"
+	void whitespace
+	void $ string "(\\"
+	void whitespace
+	wtf <- hstoken
+	void whitespace
+	void $ string "->"
+	void whitespace
+	renderer <- hstoken
+	void whitespace
+	void $ string wtf
+	void whitespace
 	return $ "(toHtml (flip " ++ renderer ++ " "
   where
 	whitespace :: Parser String
 	whitespace = many $ oneOf " \t\r\n"
 
-	token :: Parser String
-	token = many1 $ satisfy isAlphaNum <|> oneOf "_"
+	hstoken :: Parser String
+	hstoken = many1 $ satisfy isAlphaNum <|> oneOf "_"
+
+{- Use exported symbol. -}
+text_builder_hack :: String -> String
+text_builder_hack = replace "Data.Text.Lazy.Builder.Internal.fromText" "Data.Text.Lazy.Builder.fromText"
 
 {- Given a Parser that finds strings it wants to modify,
  - and returns the modified string, does a mass 
@@ -521,8 +597,8 @@ yesod_url_render_hack = parsecAndReplace $ do
  - Rather slow, but crazy powerful. -}
 parsecAndReplace :: Parser String -> String -> String
 parsecAndReplace p s = case parse find "" s of
-	Left e -> s
-	Right l -> concatMap (either (\c -> [c]) id) l
+	Left _e -> s
+	Right l -> concatMap (either return id) l
   where
   	find :: Parser [Either Char String]
 	find = many $ try (Right <$> p) <|> (Left <$> anyChar)
