@@ -8,18 +8,18 @@
 module Command.AddUnused where
 
 import Common.Annex
-import Logs.Unused
 import Logs.Location
 import Command
 import qualified Command.Add
+import Command.Unused (withUnusedMaps, UnusedMaps(..), startUnused)
 import Types.Key
 
 def :: [Command]
 def = [notDirect $ command "addunused" (paramRepeating paramNumRange)
 	seek SectionMaintenance "add back unused files"]
 
-seek :: [CommandSeek]
-seek = [withUnusedMaps start]
+seek :: CommandSeek
+seek = withUnusedMaps start
 
 start :: UnusedMaps -> Int -> CommandStart
 start = startUnused "addunused" perform
@@ -29,7 +29,7 @@ start = startUnused "addunused" perform
 perform :: Key -> CommandPerform
 perform key = next $ do
 	logStatus key InfoPresent
-	Command.Add.addLink file key False
+	Command.Add.addLink file key Nothing
 	return True
   where
 	file = "unused." ++ key2file key
