@@ -5,18 +5,25 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module Command.Fix where
 
 import Common.Annex
 import Command
 import qualified Annex.Queue
+#ifdef WITH_CLIBS
+#ifndef __ANDROID__
+import Utility.Touch
+#endif
+#endif
 
 def :: [Command]
 def = [notDirect $ noCommit $ command "fix" paramPaths seek
 	SectionMaintenance "fix up symlinks to point to annexed content"]
 
-seek :: [CommandSeek]
-seek = [withFilesInGit $ whenAnnexed start]
+seek :: CommandSeek
+seek = withFilesInGit $ whenAnnexed start
 
 {- Fixes the symlink to an annexed file. -}
 start :: FilePath -> (Key, Backend) -> CommandStart
@@ -28,9 +35,22 @@ start file (key, _) = do
 
 perform :: FilePath -> FilePath -> CommandPerform
 perform file link = do
-	liftIO $ createDirectoryIfMissing True (parentDir file)
-	liftIO $ removeFile file
-	liftIO $ createSymbolicLink link file
+	liftIO $ do
+#ifdef WITH_CLIBS
+#ifndef __ANDROID__
+		-- preserve mtime of symlink
+		mtime <- catchMaybeIO $ TimeSpec . modificationTime
+			<$> getSymbolicLinkStatus file
+#endif
+#endif
+		createDirectoryIfMissing True (parentDir file)
+		removeFile file
+		createSymbolicLink link file
+#ifdef WITH_CLIBS
+#ifndef __ANDROID__
+		maybe noop (\t -> touch file t False) mtime
+#endif
+#endif
 	next $ cleanup file
 
 cleanup :: FilePath -> CommandCleanup
