@@ -15,7 +15,6 @@ import Annex.Content
 import qualified Command.Add
 import Logs.Web
 import Logs.Location
-import Config
 import Utility.CopyFile
 
 def :: [Command]
@@ -23,8 +22,8 @@ def = [notDirect $ command "rekey"
 	(paramOptional $ paramRepeating $ paramPair paramPath paramKey)
 	seek SectionPlumbing "change keys used for files"]
 
-seek :: [CommandSeek]
-seek = [withPairs start]
+seek :: CommandSeek
+seek = withPairs start
 
 start :: (FilePath, String) -> CommandStart
 start (file, keyname) = ifAnnexed file go stop
@@ -47,18 +46,14 @@ perform file oldkey newkey = do
 			return True
 	next $ cleanup file oldkey newkey
 
-{- Make a hard link to the old key content, to avoid wasting disk space. -}
+{- Make a hard link to the old key content (when supported),
+ - to avoid wasting disk space. -}
 linkKey :: Key -> Key -> Annex Bool
 linkKey oldkey newkey = getViaTmpUnchecked newkey $ \tmp -> do
 	src <- calcRepo $ gitAnnexLocation oldkey
-	ifM (liftIO $ doesFileExist tmp)
+	liftIO $ ifM (doesFileExist tmp)
 		( return True
-		, ifM crippledFileSystem
-			( liftIO $ copyFileExternal src tmp
-			, do
-				liftIO $ createLink src tmp
-				return True
-			)
+		, createLinkOrCopy src tmp
 		)
 
 cleanup :: FilePath -> Key -> Key -> CommandCleanup
@@ -71,6 +66,6 @@ cleanup file oldkey newkey = do
 
 	-- Update symlink to use the new key.
 	liftIO $ removeFile file
-	Command.Add.addLink file newkey True
+	Command.Add.addLink file newkey Nothing
 	logStatus newkey InfoPresent
 	return True
