@@ -12,16 +12,16 @@ import Command
 import Annex.Content
 import Annex
 import Utility.Rsync
-import Logs.Transfer
-import qualified Fields
+import Annex.Transfer
+import qualified CmdLine.GitAnnexShell.Fields as Fields
 import Utility.Metered
 
 def :: [Command]
 def = [noCommit $ command "sendkey" paramKey seek
 	SectionPlumbing "runs rsync in server mode to send content"]
 
-seek :: [CommandSeek]
-seek = [withKeys start]
+seek :: CommandSeek
+seek = withKeys start
 
 start :: Key -> CommandStart
 start key = do
@@ -46,6 +46,4 @@ fieldTransfer direction key a = do
 	ok <- maybe (a $ const noop)
 		(\u -> runTransfer (Transfer direction (toUUID u) key) afile noRetry a)
 		=<< Fields.getField Fields.remoteUUID
-	if ok
-		then liftIO exitSuccess
-		else liftIO exitFailure
+	liftIO $ exitBool ok
