@@ -18,10 +18,10 @@ def =
 	, c "edit" "same as unlock"
 	]
   where
-	c n = command n paramPaths seek
+	c n = notDirect . command n paramPaths seek SectionCommon
 
-seek :: [CommandSeek]
-seek = [withFilesInGit $ whenAnnexed start]
+seek :: CommandSeek
+seek = withFilesInGit $ whenAnnexed start
 
 {- The unlock subcommand replaces the symlink with a copy of the file's
  - content. -}
@@ -35,16 +35,16 @@ perform dest key = do
 	unlessM (inAnnex key) $ error "content not present"
 	unlessM (checkDiskSpace Nothing key 0) $ error "cannot unlock"
 
-	src <- inRepo $ gitAnnexLocation key
-	tmpdest <- fromRepo $ gitAnnexTmpLocation key
+	src <- calcRepo $ gitAnnexLocation key
+	tmpdest <- fromRepo $ gitAnnexTmpObjectLocation key
 	liftIO $ createDirectoryIfMissing True (parentDir tmpdest)
 	showAction "copying"
-	ok <- liftIO $ copyFileExternal src tmpdest
-        if ok
-                then do
+	ifM (liftIO $ copyFileExternal src tmpdest)
+		( do
 			liftIO $ do
 				removeFile dest
 				moveFile tmpdest dest
 			thawContent dest
 			next $ return True
-                else error "copy failed!"
+		, error "copy failed!"
+		)
