@@ -10,19 +10,17 @@ module Command.Upgrade where
 import Common.Annex
 import Command
 import Upgrade
-import Annex.Version
 
 def :: [Command]
 def = [dontCheck repoExists $ -- because an old version may not seem to exist
 	command "upgrade" paramNothing seek
 		SectionMaintenance "upgrade repository layout"]
 
-seek :: [CommandSeek]
-seek = [withNothing start]
+seek :: CommandSeek
+seek = withNothing start
 
 start :: CommandStart
 start = do
 	showStart "upgrade" "."
-	r <- upgrade
-	setVersion defaultVersion
+	r <- upgrade False
 	next $ next $ return r
