@@ -20,19 +20,23 @@ import Assistant.Install
 import Annex.Environment
 import Utility.WebApp
 import Utility.Daemon (checkDaemon)
-import Init
+#ifdef __ANDROID__
+import Utility.Env
+#endif
+import Annex.Init
 import qualified Git
 import qualified Git.Config
 import qualified Git.CurrentRepo
 import qualified Annex
 import Config.Files
-import qualified Option
+import Upgrade
+import Annex.Version
 
 import Control.Concurrent
 import Control.Concurrent.STM
 import System.Process (env, std_out, std_err)
 import Network.Socket (HostName)
-import System.Environment
+import System.Environment (getArgs)
 
 def :: [Command]
 def = [ withOptions [listenOption] $
@@ -40,40 +44,51 @@ def = [ withOptions [listenOption] $
 	command "webapp" paramNothing seek SectionCommon "launch webapp"]
 
 listenOption :: Option
-listenOption = Option.field [] "listen" paramAddress
+listenOption = fieldOption [] "listen" paramAddress
 	"accept connections to this address"
 
-seek :: [CommandSeek]
-seek = [withField listenOption return $ \listenhost ->
-	withNothing $ start listenhost]
+seek :: CommandSeek
+seek ps = do
+	listenhost <- getOptionField listenOption return
+	withNothing (start listenhost) ps
 
 start :: Maybe HostName -> CommandStart
 start = start' True
 
 start' :: Bool -> Maybe HostName -> CommandStart
 start' allowauto listenhost = do
-	liftIO $ ensureInstalled
-	ifM isInitialized ( go , auto )
+	liftIO ensureInstalled
+	ifM isInitialized 
+		( go
+		, auto
+		)
 	stop
   where
 	go = do
+		cannotrun <- needsUpgrade . fromMaybe (error "no version") =<< getVersion
 		browser <- fromRepo webBrowser
 		f <- liftIO . absPath =<< fromRepo gitAnnexHtmlShim
+		listenhost' <- if isJust listenhost
+			then pure listenhost
+			else annexListen <$> Annex.getGitConfig
 		ifM (checkpid <&&> checkshim f)
 			( if isJust listenhost
 				then error "The assistant is already running, so --listen cannot be used."
 				else do
 					url <- liftIO . readFile
 						=<< fromRepo gitAnnexUrlFile
-					liftIO $ openBrowser browser f url Nothing Nothing
-			, startDaemon True True listenhost $ Just $ 
-				\origout origerr url htmlshim ->
-					if isJust listenhost
-						then maybe noop (`hPutStrLn` url) origout
-						else openBrowser browser htmlshim url origout origerr
+					liftIO $ if isJust listenhost'
+						then putStrLn url
+						else liftIO $ openBrowser browser f url Nothing Nothing
+			, do
+				startDaemon True True Nothing cannotrun listenhost' $ Just $ 
+					\origout origerr url htmlshim ->
+						if isJust listenhost'
+							then maybe noop (`hPutStrLn` url) origout
+							else openBrowser browser htmlshim url origout origerr
 			)
 	auto
-		| allowauto = liftIO startNoRepo
+		| allowauto = liftIO $ startNoRepo []
 		| otherwise = do
 			d <- liftIO getCurrentDirectory
 			error $ "no git repository in " ++ d
@@ -84,8 +99,8 @@ start' allowauto listenhost = do
 
 {- When run without a repo, start the first available listed repository in
  - the autostart file. If not, it's our first time being run! -}
-startNoRepo :: IO ()
-startNoRepo = do
+startNoRepo :: CmdParams -> IO ()
+startNoRepo _ = do
 	-- FIXME should be able to reuse regular getopt, but 
 	-- it currently runs in the Annex monad.
 	args <- getArgs
@@ -98,8 +113,11 @@ startNoRepo = do
 		(d:_) -> do
 			setCurrentDirectory d
 			state <- Annex.new =<< Git.CurrentRepo.get
-			void $ Annex.eval state $ doCommand $
-				start' False listenhost
+			void $ Annex.eval state $ do
+				whenM (fromRepo Git.repoIsLocalBare) $
+					error $ d ++ " is a bare git repository, cannot run the webapp in it"
+				callCommandAction $
+					start' False listenhost
 
 {- Run the webapp without a repository, which prompts the user, makes one,
  - changes to it, starts the regular assistant, and redirects the
@@ -130,8 +148,9 @@ firstRun listenhost = do
 	let callback a = Just $ a v
 	runAssistant d $ do
 		startNamedThread urlrenderer $
-			webAppThread d urlrenderer True listenhost
+			webAppThread d urlrenderer True Nothing
 				(callback signaler)
+				listenhost
 				(callback mainthread)
 		waitNamedThreads
   where
@@ -144,7 +163,8 @@ firstRun listenhost = do
 			hFlush stdout
 			go
 		| otherwise = do
-			browser <- maybe Nothing webBrowser <$> Git.Config.global
+			browser <- maybe Nothing webBrowser
+				<$> catchDefaultIO Nothing Git.Config.global
 			openBrowser browser htmlshim url Nothing Nothing
 			go
 	  where
@@ -152,27 +172,38 @@ firstRun listenhost = do
 			_wait <- takeMVar v
 			state <- Annex.new =<< Git.CurrentRepo.get
 			Annex.eval state $
-				startDaemon True True listenhost $ Just $
+				startDaemon True True Nothing Nothing listenhost $ Just $
 					sendurlback v
-	sendurlback v _origout _origerr url _htmlshim = putMVar v url
+	sendurlback v _origout _origerr url _htmlshim = do
+		recordUrl url
+		putMVar v url
+
+recordUrl :: String -> IO ()
+#ifdef __ANDROID__
+{- The Android app has a menu item that opens the url recorded
+ - in this file. -}
+recordUrl url = writeFile "/sdcard/git-annex.home/.git-annex-url" url
+#else
+recordUrl _ = noop
+#endif
 
 openBrowser :: Maybe FilePath -> FilePath -> String -> Maybe Handle -> Maybe Handle -> IO ()
-#ifdef __ANDROID__
-openBrowser mcmd htmlshim realurl outh errh = do
+#ifndef __ANDROID__
+openBrowser mcmd htmlshim _realurl outh errh = runbrowser
 #else
-openBrowser mcmd htmlshim _realurl outh errh = do
+openBrowser mcmd htmlshim realurl outh errh = do
+	recordUrl url
+	{- Android's `am` command does not work reliably across the
+	 - wide range of Android devices. Intead, FIFO should be set to 
+	 - the filename of a fifo that we can write the URL to. -}
+	v <- getEnv "FIFO"
+	case v of
+		Nothing -> runbrowser
+		Just f -> void $ forkIO $ do
+			fd <- openFd f WriteOnly Nothing defaultFileFlags
+			void $ fdWrite fd url
+			closeFd fd
 #endif
-	hPutStrLn (fromMaybe stdout outh) $ "Launching web browser on " ++ url
-	hFlush stdout
-	environ <- cleanEnvironment
-	(_, _, _, pid) <- createProcess p
-		{ env = environ
-		, std_out = maybe Inherit UseHandle outh
-		, std_err = maybe Inherit UseHandle errh
-		}
-	exitcode <- waitForProcess pid
-	unless (exitcode == ExitSuccess) $
-		hPutStrLn (fromMaybe stderr errh) "failed to start web browser"
   where
 	p = case mcmd of
 		Just cmd -> proc cmd [htmlshim]
@@ -185,6 +216,18 @@ openBrowser mcmd htmlshim _realurl outh errh = do
 #else
 	url = fileUrl htmlshim
 #endif
+	runbrowser = do
+		hPutStrLn (fromMaybe stdout outh) $ "Launching web browser on " ++ url
+		hFlush stdout
+		environ <- cleanEnvironment
+		(_, _, _, pid) <- createProcess p
+			{ env = environ
+			, std_out = maybe Inherit UseHandle outh
+			, std_err = maybe Inherit UseHandle errh
+			}
+		exitcode <- waitForProcess pid
+		unless (exitcode == ExitSuccess) $
+			hPutStrLn (fromMaybe stderr errh) "failed to start web browser"
 
 {- web.browser is a generic git config setting for a web browser program -}
 webBrowser :: Git.Repo -> Maybe FilePath
