@@ -1,4 +1,4 @@
-{-# LANGUAGE PackageImports #-}
+{-# LANGUAGE PackageImports, CPP #-}
 
 module Common (module X) where
 
@@ -11,13 +11,14 @@ import Control.Exception.Extensible as X (IOException)
 import Data.Maybe as X
 import Data.List as X hiding (head, tail, init, last)
 import Data.String.Utils as X hiding (join)
+import Data.Monoid as X
 
-import "MissingH" System.Path as X
 import System.FilePath as X
 import System.Directory as X
 import System.IO as X hiding (FilePath)
-import System.Posix.Files as X
+#ifndef mingw32_HOST_OS
 import System.Posix.IO as X
+#endif
 import System.Exit as X
 
 import Utility.Misc as X
@@ -27,7 +28,9 @@ import Utility.Process as X
 import Utility.Path as X
 import Utility.Directory as X
 import Utility.Monad as X
+import Utility.Data as X
 import Utility.Applicative as X
 import Utility.FileSystemEncoding as X
+import Utility.PosixFiles as X
 
 import Utility.PartialPrelude as X
