@@ -5,11 +5,18 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
-module Remote.Rsync (remote) where
+{-# LANGUAGE CPP #-}
 
-import qualified Data.ByteString.Lazy as L
-import qualified Data.Map as M
-import System.Posix.Process (getProcessID)
+module Remote.Rsync (
+	remote,
+	storeEncrypted,
+	retrieveEncrypted,
+	remove,
+	checkPresent,
+	withRsyncScratchDir,
+	genRsyncOpts,
+	RsyncOpts
+) where
 
 import Common.Annex
 import Types.Remote
@@ -17,22 +24,22 @@ import qualified Git
 import Config
 import Config.Cost
 import Annex.Content
+import Annex.UUID
 import Annex.Ssh
 import Remote.Helper.Special
 import Remote.Helper.Encryptable
+import Remote.Rsync.RsyncUrl
 import Crypto
 import Utility.Rsync
 import Utility.CopyFile
 import Utility.Metered
+import Utility.PID
 import Annex.Perms
+import Logs.Transfer
+import Types.Creds
 
-type RsyncUrl = String
-
-data RsyncOpts = RsyncOpts
-	{ rsyncUrl :: RsyncUrl
-	, rsyncOptions :: [CommandParam]
-	, rsyncShellEscape :: Bool
-}
+import qualified Data.ByteString.Lazy as L
+import qualified Data.Map as M
 
 remote :: RemoteType
 remote = RemoteType {
@@ -42,14 +49,15 @@ remote = RemoteType {
 	setup = rsyncSetup
 }
 
-gen :: Git.Repo -> UUID -> RemoteConfig -> RemoteGitConfig -> Annex Remote
+gen :: Git.Repo -> UUID -> RemoteConfig -> RemoteGitConfig -> Annex (Maybe Remote)
 gen r u c gc = do
 	cst <- remoteCost gc expensiveRemoteCost
-	(transport, url) <- rsyncTransport
-	let o = RsyncOpts url (transport ++ opts) escape
-	    islocal = rsyncUrlIsPath $ rsyncUrl o
-	return $ encryptableRemote c
-		(storeEncrypted o $ getGpgOpts gc)
+	(transport, url) <- rsyncTransport gc $
+		fromMaybe (error "missing rsyncurl") $ remoteAnnexRsyncUrl gc
+	let o = genRsyncOpts c gc transport url
+	let islocal = rsyncUrlIsPath $ rsyncUrl o
+	return $ Just $ encryptableRemote c
+		(storeEncrypted o $ getGpgEncParams (c,gc))
 		(retrieveEncrypted o)
 		Remote
 			{ uuid = u
@@ -62,19 +70,30 @@ gen r u c gc = do
 			, hasKey = checkPresent r o
 			, hasKeyCheap = False
 			, whereisKey = Nothing
-			, config = M.empty
+			, remoteFsck = Nothing
+			, repairRepo = Nothing
+			, config = c
 			, repo = r
 			, gitconfig = gc
 			, localpath = if islocal
 				then Just $ rsyncUrl o
 				else Nothing
 			, readonly = False
-			, globallyAvailable = not $ islocal
+			, availability = if islocal then LocallyAvailable else GloballyAvailable
 			, remotetype = remote
 			}
+
+genRsyncOpts :: RemoteConfig -> RemoteGitConfig -> [CommandParam] -> RsyncUrl -> RsyncOpts
+genRsyncOpts c gc transport url = RsyncOpts
+	{ rsyncUrl = url
+	, rsyncOptions = opts []
+	, rsyncUploadOptions = transport ++ opts (remoteAnnexRsyncUploadOptions gc)
+	, rsyncDownloadOptions = transport ++ opts (remoteAnnexRsyncDownloadOptions gc)
+	, rsyncShellEscape = M.lookup "shellescape" c /= Just "no"
+	}
   where
-	opts = map Param $ filter safe $ remoteAnnexRsyncOptions gc
-	escape = M.lookup "shellescape" c /= Just "no"
+	opts specificopts = map Param $ filter safe $
+		remoteAnnexRsyncOptions gc ++ specificopts
 	safe opt
 		-- Don't allow user to pass --delete to rsync;
 		-- that could cause it to delete other keys
@@ -82,31 +101,34 @@ gen r u c gc = do
 		| opt == "--delete" = False
 		| opt == "--delete-excluded" = False
 		| otherwise = True
-	rawurl = fromMaybe (error "missing rsyncurl") $ remoteAnnexRsyncUrl gc
-	(login,resturl) = case separate (=='@') rawurl of
-			(h, "") -> (Nothing, h)
-			(l, h)  -> (Just l, h)
-	loginopt = maybe [] (\l -> ["-l",l]) login
-	fromNull as xs | null xs = as
-		       | otherwise = xs
-	rsyncTransport = if rsyncUrlIsShell rawurl
-		then (\rsh -> return (rsyncShell rsh, resturl)) =<<
-			case fromNull ["ssh"] (remoteAnnexRsyncTransport gc) of
-				"ssh":sshopts -> do
-					let (port, sshopts') = sshReadPort sshopts
-					    host = takeWhile (/=':') resturl
-					-- Connection caching
-					(Param "ssh":) <$> sshCachingOptions
-						(host, port)
-						(map Param $ loginopt ++ sshopts')
-				"rsh":rshopts -> return $ map Param $ "rsh" :
-					loginopt ++ rshopts
-				rsh -> error $ "Unknown Rsync transport: "
-					++ unwords rsh
-		else return ([], rawurl)
 
-rsyncSetup :: UUID -> RemoteConfig -> Annex RemoteConfig
-rsyncSetup u c = do
+rsyncTransport :: RemoteGitConfig -> RsyncUrl -> Annex ([CommandParam], RsyncUrl)
+rsyncTransport gc url
+	| rsyncUrlIsShell url =
+		(\rsh -> return (rsyncShell rsh, url)) =<<
+		case fromNull ["ssh"] (remoteAnnexRsyncTransport gc) of
+			"ssh":sshopts -> do
+				let (port, sshopts') = sshReadPort sshopts
+				    userhost = takeWhile (/=':') url
+				-- Connection caching
+				(Param "ssh":) <$> sshCachingOptions
+					(userhost, port)
+					(map Param $ loginopt ++ sshopts')
+			"rsh":rshopts -> return $ map Param $ "rsh" :
+				loginopt ++ rshopts
+			rsh -> error $ "Unknown Rsync transport: "
+				++ unwords rsh
+	| otherwise = return ([], url)
+  where
+	login = case separate (=='@') url of
+		(_h, "") -> Nothing
+		(l, _)  -> Just l
+	loginopt = maybe [] (\l -> ["-l",l]) login
+	fromNull as xs = if null xs then as else xs
+
+rsyncSetup :: Maybe UUID -> Maybe CredPair -> RemoteConfig -> Annex (RemoteConfig, UUID)
+rsyncSetup mu _ c = do
+	u <- maybe (liftIO genUUID) return mu
 	-- verify configuration is sane
 	let url = fromMaybe (error "Specify rsyncurl=") $
 		M.lookup "rsyncurl" c
@@ -115,23 +137,12 @@ rsyncSetup u c = do
 	-- The rsyncurl is stored in git config, not only in this remote's
 	-- persistant state, so it can vary between hosts.
 	gitConfigSpecialRemote u c' "rsyncurl" url
-	return c'
-
-rsyncEscape :: RsyncOpts -> String -> String
-rsyncEscape o s
-	| rsyncShellEscape o && rsyncUrlIsShell (rsyncUrl o) = shellEscape s
-	| otherwise = s
-
-rsyncUrls :: RsyncOpts -> Key -> [String]
-rsyncUrls o k = map use annexHashes
-  where
-	use h = rsyncUrl o </> h k </> rsyncEscape o (f </> f)
-	f = keyFile k
+	return (c', u)
 
 store :: RsyncOpts -> Key -> AssociatedFile -> MeterUpdate -> Annex Bool
 store o k _f p = sendAnnex k (void $ remove o k) $ rsyncSend o p k False
 
-storeEncrypted :: RsyncOpts -> GpgOpts -> (Cipher, Key) -> Key -> MeterUpdate -> Annex Bool
+storeEncrypted :: RsyncOpts -> [CommandParam] -> (Cipher, Key) -> Key -> MeterUpdate -> Annex Bool
 storeEncrypted o gpgOpts (cipher, enck) k p = withTmp enck $ \tmp ->
 	sendAnnex k (void $ remove o enck) $ \src -> do
 		liftIO $ encrypt gpgOpts cipher (feedFile src) $
@@ -155,18 +166,20 @@ retrieveEncrypted o (cipher, enck) _ f p = withTmp enck $ \tmp ->
 		)
 
 remove :: RsyncOpts -> Key -> Annex Bool
-remove o k = withRsyncScratchDir $ \tmp -> liftIO $ do
-	{- Send an empty directory to rysnc to make it delete. -}
-	let dummy = tmp </> keyFile k
-	createDirectoryIfMissing True dummy
-	rsync $ rsyncOptions o ++
-		map (\s -> Param $ "--include=" ++ s) includes ++
-		[ Param "--exclude=*" -- exclude everything else
-		, Params "--quiet --delete --recursive"
-		, partialParams
-		, Param $ addTrailingPathSeparator dummy
-		, Param $ rsyncUrl o
-		]
+remove o k = do
+	ps <- sendParams
+	withRsyncScratchDir $ \tmp -> liftIO $ do
+		{- Send an empty directory to rysnc to make it delete. -}
+		let dummy = tmp </> keyFile k
+		createDirectoryIfMissing True dummy
+		rsync $ rsyncOptions o ++ ps ++
+			map (\s -> Param $ "--include=" ++ s) includes ++
+			[ Param "--exclude=*" -- exclude everything else
+			, Params "--quiet --delete --recursive"
+			, partialParams
+			, Param $ addTrailingPathSeparator dummy
+			, Param $ rsyncUrl o
+			]
   where
 	{- Specify include rules to match the directories where the
 	 - content could be. Note that the parent directories have
@@ -200,14 +213,23 @@ checkPresent r o k = do
 partialParams :: CommandParam
 partialParams = Params "--partial --partial-dir=.rsync-partial"
 
+{- When sending files from crippled filesystems, the permissions can be all
+ - messed up, and it's better to use the default permissions on the
+ - destination. -}
+sendParams :: Annex [CommandParam]
+sendParams = ifM crippledFileSystem
+	( return [rsyncUseDestinationPermissions]
+	, return []
+	)
+
 {- Runs an action in an empty scratch directory that can be used to build
  - up trees for rsync. -}
-withRsyncScratchDir :: (FilePath -> Annex Bool) -> Annex Bool
+withRsyncScratchDir :: (FilePath -> Annex a) -> Annex a
 withRsyncScratchDir a = do
-	pid <- liftIO getProcessID
-	t <- fromRepo gitAnnexTmpDir
+	p <- liftIO getPID
+	t <- fromRepo gitAnnexTmpObjectDir
 	createAnnexDirectory t
-	let tmp = t </> "rsynctmp" </> show pid
+	let tmp = t </> "rsynctmp" </> show p
 	nuke tmp
 	liftIO $ createDirectoryIfMissing True tmp
 	nuke tmp `after` a tmp
@@ -217,25 +239,30 @@ withRsyncScratchDir a = do
 
 rsyncRetrieve :: RsyncOpts -> Key -> FilePath -> Maybe MeterUpdate -> Annex Bool
 rsyncRetrieve o k dest callback =
-	untilTrue (rsyncUrls o k) $ \u -> rsyncRemote o callback
+	showResumable $ untilTrue (rsyncUrls o k) $ \u -> rsyncRemote Download o callback
 		-- use inplace when retrieving to support resuming
 		[ Param "--inplace"
 		, Param u
-		, Param dest
+		, File dest
 		]
 
-rsyncRemote :: RsyncOpts -> (Maybe MeterUpdate) -> [CommandParam] -> Annex Bool
-rsyncRemote o callback params = do
+showResumable :: Annex Bool -> Annex Bool
+showResumable a = ifM a
+	( return True
+	, do
+		showLongNote "rsync failed -- run git annex again to resume file transfer"
+		return False
+	)
+
+rsyncRemote :: Direction -> RsyncOpts -> Maybe MeterUpdate -> [CommandParam] -> Annex Bool
+rsyncRemote direction o callback params = do
 	showOutput -- make way for progress bar
-	ifM (liftIO $ (maybe rsync rsyncProgress callback) ps)
-		( return True
-		, do
-			showLongNote "rsync failed -- run git annex again to resume file transfer"
-			return False
-		)
+	liftIO $ (maybe rsync rsyncProgress callback) $
+		opts ++ [Params "--progress"] ++ params
   where
-	defaultParams = [Params "--progress"]
-	ps = rsyncOptions o ++ defaultParams ++ params
+	opts
+		| direction == Download = rsyncDownloadOptions o
+		| otherwise = rsyncUploadOptions o
 
 {- To send a single key is slightly tricky; need to build up a temporary
  - directory structure to pass to rsync so it can create the hash
@@ -243,7 +270,7 @@ rsyncRemote o callback params = do
  -
  - This would not be necessary if the hash directory structure used locally
  - was always the same as that used on the rsync remote. So if that's ever
- - unified, this gets nicer. Especially in the crippled filesystem case.
+ - unified, this gets nicer.
  - (When we have the right hash directory structure, we can just
  - pass --include=X --include=X/Y --include=X/Y/file --exclude=*)
  -}
@@ -251,22 +278,18 @@ rsyncSend :: RsyncOpts -> MeterUpdate -> Key -> Bool -> FilePath -> Annex Bool
 rsyncSend o callback k canrename src = withRsyncScratchDir $ \tmp -> do
 	let dest = tmp </> Prelude.head (keyPaths k)
 	liftIO $ createDirectoryIfMissing True $ parentDir dest
-	ok <- if canrename
+	ok <- liftIO $ if canrename
 		then do
-			liftIO $ renameFile src dest
+			rename src dest
 			return True
-		else ifM crippledFileSystem
-			( liftIO $ copyFileExternal src dest
-			, do
-				liftIO $ createLink src dest
-				return True
-			)
+		else createLinkOrCopy src dest
+	ps <- sendParams
 	if ok
-		then rsyncRemote o (Just callback)
+		then showResumable $ rsyncRemote Upload o (Just callback) $ ps ++
 			[ Param "--recursive"
 			, partialParams
 			-- tmp/ to send contents of tmp dir
-			, Param $ addTrailingPathSeparator tmp
+			, File $ addTrailingPathSeparator tmp
 			, Param $ rsyncUrl o
 			]
 		else return False
