@@ -5,17 +5,23 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module Utility.LogFile where
 
 import Common
 
-import System.Posix
+#ifndef mingw32_HOST_OS
+import System.Posix.Types
+#endif
 
+#ifndef mingw32_HOST_OS
 openLog :: FilePath -> IO Fd
 openLog logfile = do
 	rotateLog logfile
 	openFd logfile WriteOnly (Just stdFileMode)
 		defaultFileFlags { append = True }
+#endif
 
 rotateLog :: FilePath -> IO ()
 rotateLog logfile = go 0
@@ -24,7 +30,7 @@ rotateLog logfile = go 0
 		| num > maxLogs = return ()
 		| otherwise = whenM (doesFileExist currfile) $ do
 			go (num + 1)
-			renameFile currfile nextfile
+			rename currfile nextfile
 	  where
 		currfile = filename num
 		nextfile = filename (num + 1)
@@ -43,6 +49,7 @@ listLogs logfile = filterM doesFileExist $ reverse $
 maxLogs :: Int
 maxLogs = 9
 
+#ifndef mingw32_HOST_OS
 redirLog :: Fd -> IO ()
 redirLog logfd = do
 	mapM_ (redir logfd) [stdOutput, stdError]
@@ -52,3 +59,4 @@ redir :: Fd -> Fd -> IO ()
 redir newh h = do
 	closeFd h
 	void $ dupTo newh h
+#endif
