@@ -1,6 +1,6 @@
 {- various rsync stuff
  -
- - Copyright 2010-2012 Joey Hess <joey@kitenet.net>
+ - Copyright 2010-2013 Joey Hess <joey@kitenet.net>
  -
  - Licensed under the GNU GPL version 3 or higher.
  -}
@@ -45,8 +45,20 @@ rsyncServerParams =
 	, Params "-e.Lsf ."
 	]
 
+rsyncUseDestinationPermissions :: CommandParam
+rsyncUseDestinationPermissions = Param "--chmod=ugo=rwX"
+
 rsync :: [CommandParam] -> IO Bool
-rsync = boolSystem "rsync"
+rsync = boolSystem "rsync" . rsyncParamsFixup
+
+{- On Windows, rsync is from Cygwin, and expects to get Cygwin formatted
+ - paths to files. (It thinks that C:foo refers to a host named "C").
+ - Fix up all Files in the Params appropriately. -}
+rsyncParamsFixup :: [CommandParam] -> [CommandParam]
+rsyncParamsFixup = map fixup
+  where
+  	fixup (File f) = File (toCygPath f)
+	fixup p = p
 
 {- Runs rsync, but intercepts its progress output and updates a meter.
  - The progress output is also output to stdout. 
@@ -55,14 +67,15 @@ rsync = boolSystem "rsync"
  -}
 rsyncProgress :: MeterUpdate -> [CommandParam] -> IO Bool
 rsyncProgress meterupdate params = do
-	r <- withHandle StdoutHandle createProcessSuccess p (feedprogress 0 [])
+	r <- catchBoolIO $ 
+		withHandle StdoutHandle createProcessSuccess p (feedprogress 0 [])
 	{- For an unknown reason, piping rsync's output like this does
 	 - causes it to run a second ssh process, which it neglects to wait
 	 - on. Reap the resulting zombie. -}
 	reapZombies
 	return r
   where
-	p = proc "rsync" (toCommand params)
+	p = proc "rsync" (toCommand $ rsyncParamsFixup params)
 	feedprogress prev buf h = do
 		s <- hGetSomeString h 80
 		if null s
@@ -111,6 +124,9 @@ rsyncUrlIsPath s
  - after the \r is the number of bytes processed. After the number,
  - there must appear some whitespace, or we didn't get the whole number,
  - and return the \r and part we did get, for later processing.
+ -
+ - In some locales, the number will have one or more commas in the middle
+ - of it.
  -}
 parseRsyncProgress :: String -> (Maybe Integer, String)
 parseRsyncProgress = go [] . reverse . progresschunks
@@ -129,7 +145,7 @@ parseRsyncProgress = go [] . reverse . progresschunks
 	parsebytes s = case break isSpace s of
 		([], _) -> Nothing
 		(_, []) -> Nothing
-		(b, _) -> readish b
+		(b, _) -> readish $ filter (/= ',') b
 
 {- Filters options to those that are safe to pass to rsync in server mode,
  - without causing it to eg, expose files. -}
